
clear all;

% prepare your data here
% ............

% X: data matrix, DxN(D is the dimension, N is the number of samples)
fea = NormalizeFea(fea);
X = fea';
[nFea,nSmp] = size(X);
nClass = length(unique(gnd));

%% sparse manifold fitting, this is a unsupervised process
% lambda: regularization parameter of the sparse manifold fitting
lambda = 1;
% weight matrix used for identifying the submanifold
W = zeros(nSmp,nSmp);
% 'KMax' nearset neighbors to carry out l1 minimization, n/(4c)
KMax = 20;
% construct the 'Y': local direction matrix
Dist = EuDist2(X');
% solving the 'sparse manifold fitting' optimization problem for each data
% point
for i = 1:nSmp
    [~,ids] = sort(Dist(:,i),'ascend');
    ids = ids(1:KMax);
    
    y = X(:,ids(1));
    Y = X(:,ids);
    Y(:,1) = [];
    Y = Y - repmat(y,1,KMax-1);
    v = Dist(ids,i);
    v(1) = [];
    for j = 1:KMax-1
        Y(:,j) = Y(:,j) ./ v(j);
    end
    
    % solving the Lasso-type optimization using ADMM framework
    c = admm_vec_func(Y,v./sum(v),lambda);
    
    W(ids(2:KMax),i) = abs(c./v) / sum(abs(c./v));
end
W = processC(W,0.9);
figure; imagesc(W)

%% semi-supervised learning
% enhanced low-rank representation via sparse manifold fitting
para = [];
para.lambda = 10;
para.alpha  = 0.01;
para.beta   = 0.01;
rho = 1.1;
[Z,~,~] = MLRR(X,W,para,rho,1);
S = Z;
% for i = 1:size(S,2)
%     S(:,i) = S(:,i)/norm(S(:,i));
% end
S(S<0) = 0;
S = processC(S,0.8);
S = (S + S')/2;
% semi-supervised learning
% 'percentLib': percent of instances per class
percentLib = [0.1 0.2 0.3 0.4 0.5 0.6];
nRounds = 50;
acc = zeros(nRounds,length(percentLib));

load('randIdx_YaleB.mat');
for m = 1:length(percentLib)
    percent = percentLib(m);
    disp(num2str(m));
    % 50 independent runs
    for n = 1:nRounds
        % construct 'Y'
        Y = zeros(nSmp,nClass);
        count = 0;
        for i = 1:nClass
            index = find(gnd == i);
            % idx   = randperm(length(index));
            idx = randIdx_YaleB(count+1:count+length(index),n);
            % randomly select 'perLabeled' images as labeled
            perLabeled = ceil(percent*length(index));
            for j = 1:perLabeled
                % Y(index(idx(j)),i) = 1;
                Y(idx(j),i) = 1;
            end
            count = count + length(index);
        end
        % call 'LabelPropagation_L2' to obtain the 'Q'
        Q = LabelPropagation_L2(S,Y);
        % calculate the accuracy
        [~,predict] = max(Q,[],2);
        
        index1 = find(max(Y,[],2) == 1);
        predict(index1) = [];
        temp = gnd;
        temp(index1) = [];
        acc(n,m) = length(find(predict == temp))/length(predict);
    end   
end
% show the result
disp(['accuracy:' num2str(mean(acc))]);
disp(['std-dev:' num2str(std(acc))]);