function F = LabelPropagation_L2(A, Y)
% Semi-supervised Learning using L21-norm minimization
% min_F  0.5*\sum_ij A_ij*||Fi-Fj||_2^2 + tr(F-Y)'*U*(F-Y)

% A: similarity matrix on graph, n*n matrix
% Y: label matrix, n*c matrix. 
%       For the labeled data xi, Yij = 1 if xi belongs to the j-th class, and Yij = 0 otherwise. 
%       For the unlabeled data xi, Yij=0 for any j
% F: predicted label matrix, n*c matrix
% obj: objective values during the iterations

D = diag(sum(A,2));
n = size(A,1);
u = zeros(n,1);
y = sum(Y,2);
u(y==1) = 1;
U = diag(u);

L = D - A;
%L = D^(-1/2)*L*D^(-1/2);
F = (L+U)\(U*Y);
F = F*diag(sqrt(1./diag(F'*D*F)));